require 'spec_helper'

describe RestBase::Application do
  before(:each) do
    @headers = { 'HTTP_AUTHORIZATION' => 'key1' }  
  end
  
  describe :version do
    it "should set the version to latest when not given" do
      expected_version = "version=1"
      
      get '/', {}, @headers
      
      expect(last_request.env["HTTP_ACCEPT"].include? expected_version).to be_truthy
      expect(last_response.headers["Content-Type"].include? expected_version).to be_truthy
    end
    
    it "should get the version 1 version" do
      expected_version = "version=1"
      @headers['HTTP_ACCEPT'] = "application/json;#{expected_version}"
      
      get '/', {}, @headers
      
      expect(last_request.env["HTTP_ACCEPT"].include? expected_version).to be_truthy
      expect(last_response.headers["Content-Type"].include? expected_version).to be_truthy
      expect(last_response.body).to eq({:result => { :gotten => "success" }}.to_json)
    end
    
    it "should get the version 2 version" do
      expected_version = "version=2"
      @headers['HTTP_ACCEPT'] = expected_version
      
      get '/', {}, @headers
      
      expect(last_request.env["HTTP_ACCEPT"].include? expected_version).to be_truthy
      expect(last_response.headers["Content-Type"].include? expected_version).to be_truthy
      expect(last_response.body).to eq({:result => { :gotten => "success 2" }}.to_json)
    end
  end
  
  describe :method do
    it "should go through the before filter" do
      get '/method_filter', {}, @headers
      
      expect(last_response.body).to eq({:result => ["Hit Before Filter", "Response Filtered"]}.to_json)
    end
    
    it "should not go through the before filter" do
      put '/method_filter', {}, @headers
      
      expect(last_response.body).to eq({:result => ["Response Filtered"]}.to_json)
    end
  end
  
  describe :authorization do
    it "should try to authorize" do
      get '/', {}, @headers
    end
  end
  
  describe :status do
    it "should return status code 200" do
      get '/', {}, @headers
      
      expect(last_response.status).to eq(200)
    end
  end
  
  describe :empty_response do
    it "should return an empty string response" do
      get '/empty', {}, @headers
      
      expect(last_response.body).to eq({}.to_json)
    end
  end
  
  describe :no_content do
    it "should return a no content response with empty body" do
      get '/nocontent', {}, @headers
      
      expect(last_response.status).to eq(204)
      expect(last_response.body).to be_empty
    end
    
    it "should return a created response with empty body" do
      get '/created', {}, @headers
      
      expect(last_response.status).to eq(201)
      expect(last_response.body).to be_empty
    end
    
    it "shoud return a created response with body" do
      get '/created_with_body', {}, @headers
      
      expect(last_response.status).to eq(201)
      expect(last_response.body).to eq({:result => {:created => true}}.to_json)
    end
    
    it "should return a not modified response with empty body" do
      get '/notmodified', {}, @headers
      
      expect(last_response.status).to eq(304)
      expect(last_response.body).to be_empty
    end
    
    it "should delete and return a no content response with empty body" do
      delete '/nocontent', {}, @headers
      
      expect(last_response.status).to eq(204)
      expect(last_response.body).to be_empty
    end
  end

  describe :forensics do
    before(:each) do
      @expected_hash = {:result => ["Forensics Test"], :forensics => ["Added Forensics"]}
      @headers['Forensics'] = 'TRUE'
    end
    
    it "should add forensics to body" do
      get '/forensics', {}, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should not add forensics to body" do
      @headers.delete('Forensics')
      get '/forensics', {}, @headers
      
      @expected_hash.delete(:forensics)
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should not add nil/empty forensics to body" do
      get '/forensics_empty', {}, @headers
      
      @expected_hash.delete(:forensics)
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
  end
    
  describe :json do
    before(:each) do
      @posted_body =  { :success => true }
      
      @expected_hash = {:result => { :posted => @posted_body }}
    end
    
    it "should get json" do
      @expected_hash = {:result => { :gotten => "success" }}
      
      get '/', {}, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should set the request body to json" do  
      @headers['CONTENT_TYPE'] = 'application/json'
      
      post '/', @posted_body.to_json, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should post xml and return json" do
      @headers['CONTENT_TYPE'] = 'text/xml'
      post '/', @posted_body.to_xml(:root => :request), @headers
      
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should post empty json and return json" do
      post '/', "", @headers
      
      @expected_hash[:result][:posted] = {}
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
    
    it "should post json with json string as body" do
      post '/', @posted_body.to_json, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
  
    it "should fail with bad json" do
      post '/', "}", @headers
      
      @expected_hash.delete(:result)
      @expected_hash[:errors] = ["Invalid Request Body"]
      
      expect(last_response.status).to eq(500)
      expect(last_response.body).to eq(@expected_hash.to_json)
    end
  end
  
  describe :xml do
    before(:each) do
      @posted_body =  { :success => true }
      
      @expected_hash = {:result => { :posted => @posted_body }}
    end
    
    it "should get xml" do
      @headers['HTTP_ACCEPT'] = 'text/xml'
      @expected_hash = {:result => { :gotten => 'success'} }
      
      get '/', {}, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_xml(:root => :response))
    end
    
    it "should set the request body to xml" do  
      @headers['CONTENT_TYPE'] = 'text/xml'
      @headers['HTTP_ACCEPT'] = 'text/xml'
      post '/', @posted_body.to_xml(:root => :request), @headers
      
      expect(last_response.body).to eq(@expected_hash.to_xml(:root => :response))
    end
    
    it "should post json and return xml" do
      @headers['CONTENT_TYPE'] = 'application/json'
      @headers['HTTP_ACCEPT'] = 'text/xml'
      post '/', @posted_body.to_json, @headers
      
      expect(last_response.body).to eq(@expected_hash.to_xml(:root => :response))
    end
    
    it "should post json with json string as body" do
      @headers['CONTENT_TYPE'] = 'text/xml'
      @headers['HTTP_ACCEPT'] = 'text/xml'
      post '/', @posted_body.to_xml(:root => :request), @headers
      
      expect(last_response.body).to eq(@expected_hash.to_xml(:root => :response))
    end
    
    it "should fail on bad xml" do
      @headers['CONTENT_TYPE'] = 'text/xml'
      @headers['HTTP_ACCEPT'] = 'text/xml'
      
      @expected_hash.delete(:result)
      @expected_hash[:errors] = ["Invalid Request Body"]
      
      post '/', "<BadXML>", @headers
      
      expect(last_response.status).to eq(500)
      expect(last_response.body).to eq(@expected_hash.to_xml(:root => :response))
    end
  end
end
